@@ -24,6 +24,7 @@
 
 #include <Libkleo/MessageBox>
 #include <Libkleo/Classify>
+#include <Libkleo/SystemInfo>
 
 #include <gpgme++/key.h>
 #include <gpgme++/decryptionresult.h>
@@ -236,9 +237,18 @@ ResultItemWidget::ResultItemWidget(const std::shared_ptr<const Task::Result> &re
     const QColor color = colorForVisualCode(d->m_result->code());
     const QColor txtColor = txtColorForVisualCode(d->m_result->code());
     const QColor linkColor = txtColor;
-    const QString styleSheet = QStringLiteral("QFrame,QLabel { background-color: %1; margin: 0px; }"
-                                              "QFrame#resultFrame{ border-color: %2; border-style: solid; border-radius: 3px; border-width: 1px }"
-                                              "QLabel { color: %3; padding: 5px; border-radius: 3px }").arg(color.name()).arg(color.darker(150).name()).arg(txtColor.name());
+    const QString styleSheet = SystemInfo::isHighContrastModeActive()
+        ? QStringLiteral(
+            "QFrame,QLabel { margin: 0px; }"
+            "QFrame#resultFrame{ border-style: solid; border-radius: 3px; border-width: 1px }"
+            "QLabel { padding: 5px; border-radius: 3px }")
+        : QStringLiteral(
+              "QFrame,QLabel { background-color: %1; margin: 0px; }"
+              "QFrame#resultFrame{ border-color: %2; border-style: solid; border-radius: 3px; border-width: 1px }"
+              "QLabel { color: %3; padding: 5px; border-radius: 3px }")
+              .arg(color.name())
+              .arg(color.darker(150).name())
+              .arg(txtColor.name());
     auto topLayout = new QVBoxLayout(this);
     auto frame = new QFrame;
     frame->setObjectName(QStringLiteral("resultFrame"));
