@@ -13,12 +13,15 @@
 
 #include "expirydialog.h"
 
+#include "utils/gui-helper.h"
 #include "utils/qt-cxx20-compat.h"
 
+#include <Libkleo/Formatting>
+
+#include <KDateComboBox>
 #include <KLocalizedString>
 #include <KStandardGuiItem>
 
-#include <QCalendarWidget>
 #include <QCheckBox>
 #include <QComboBox>
 #include <QDate>
@@ -59,6 +62,23 @@ static QDate date_by_amount_and_unit(int inAmount, int inUnit)
     return QDate();
 }
 
+static QString accessibleValidityDuration(int amount, Period unit)
+{
+    switch (unit) {
+    case Days:
+        return i18np("Valid for %1 day", "Valid for %1 days", amount);
+    case Weeks:
+        return i18np("Valid for %1 week", "Valid for %1 weeks", amount);
+    case Months:
+        return i18np("Valid for %1 month", "Valid for %1 months", amount);
+    case Years:
+        return i18np("Valid for %1 year", "Valid for %1 years", amount);
+    default:
+        Q_ASSERT(!"invalid unit");
+    }
+    return {};
+}
+
 // these calculations should be precise enough for the forseeable future...
 static const double DAYS_IN_GREGORIAN_YEAR = 365.2425;
 
@@ -74,12 +94,6 @@ static int yearsBetween(const QDate &d1, const QDate &d2)
     return qRound(days / DAYS_IN_GREGORIAN_YEAR);
 }
 
-auto radioButtonSize(const QRadioButton *radioButton)
-{
-    QStyleOptionButton opt;
-    return radioButton->style()->sizeFromContents(QStyle::CT_RadioButton, &opt, QSize(), radioButton);
-}
-
 }
 
 class ExpiryDialog::Private
@@ -102,18 +116,8 @@ public:
 #endif
         connect(ui.inCB, qOverload<int>(&QComboBox::currentIndexChanged),
                 q, [this] () { slotInUnitChanged(); });
-        connect(ui.onCW, &QCalendarWidget::selectionChanged,
+        connect(ui.onCB, &KDateComboBox::dateChanged,
                 q, [this] () { slotOnDateChanged(); });
-        connect(ui.onCW, &QCalendarWidget::currentPageChanged,
-                q, [this] (int year, int month) {
-                    // We select the same day in the month when
-                    // a page is switched.
-                    auto date = ui.onCW->selectedDate();
-                    if (!date.setDate(year, month, date.day())) {
-                        date.setDate(year, month, 1);
-                    }
-                    ui.onCW->setSelectedDate(date);
-                });
 
         Q_ASSERT(ui.inCB->currentIndex() == inUnit);
     }
@@ -126,10 +130,12 @@ private:
 private:
     QDate inDate() const;
     int inAmountByDate(const QDate &date) const;
+    void setInitialFocus();
 
 private:
     ExpiryDialog::Mode mode;
     int inUnit;
+    bool initialFocusWasSet = false;
 
     struct UI {
         QRadioButton *neverRB;
@@ -137,7 +143,7 @@ private:
         QSpinBox *inSB;
         QComboBox *inCB;
         QRadioButton *onRB;
-        QCalendarWidget *onCW;
+        KDateComboBox *onCB;
         QCheckBox *updateSubkeysCheckBox;
 
         explicit UI(Mode mode, Dialogs::ExpiryDialog *qq)
@@ -152,12 +158,12 @@ private:
             {
                 auto label = new QLabel{qq};
                 label->setText(mode == Mode::UpdateIndividualSubkey ?
-                               i18n("Please select when to expire this subkey:") :
-                               i18n("Please select when to expire this certificate:"));
+                               i18n("Please select until when the subkey should be valid:") :
+                               i18n("Please select until when the certificate should be valid:"));
                 vboxLayout->addWidget(label);
             }
 
-            neverRB = new QRadioButton(i18n("Ne&ver"), mainWidget);
+            neverRB = new QRadioButton(i18n("Unlimited validity"), mainWidget);
             neverRB->setChecked(false);
 
             vboxLayout->addWidget(neverRB);
@@ -165,7 +171,7 @@ private:
             {
                 auto hboxLayout = new QHBoxLayout;
 
-                inRB = new QRadioButton{i18n("In"), mainWidget};
+                inRB = new QRadioButton{i18n("Valid for:"), mainWidget};
                 inRB->setChecked(false);
 
                 hboxLayout->addWidget(inRB);
@@ -192,21 +198,18 @@ private:
                 vboxLayout->addLayout(hboxLayout);
             }
 
-            onRB = new QRadioButton{i18n("On this da&y:"), mainWidget};
-            onRB->setChecked(true);
-
-            vboxLayout->addWidget(onRB);
-
             {
                 auto hboxLayout = new QHBoxLayout;
 
-                hboxLayout->addSpacing(radioButtonSize(onRB).width());
+                onRB = new QRadioButton{i18n("Valid until:"), mainWidget};
+                onRB->setChecked(true);
 
-                onCW = new QCalendarWidget{mainWidget};
-                onCW->setGridVisible(true);
-                onCW->setMinimumDate(QDate::currentDate().addDays(1));
+                hboxLayout->addWidget(onRB);
 
-                hboxLayout->addWidget(onCW);
+                onCB = new KDateComboBox{mainWidget};
+                onCB->setMinimumDate(QDate::currentDate().addDays(1));
+
+                hboxLayout->addWidget(onCB);
 
                 hboxLayout->addStretch(1);
 
@@ -214,7 +217,7 @@ private:
             }
 
             {
-                updateSubkeysCheckBox = new QCheckBox{i18n("Also update the expiry of the subkeys"), qq};
+                updateSubkeysCheckBox = new QCheckBox{i18n("Also update the validity period of the subkeys"), qq};
 #ifdef QGPGME_SUPPORTS_CHANGING_EXPIRATION_OF_COMPLETE_KEY
                 updateSubkeysCheckBox->setVisible(mode == Mode::UpdateCertificateWithSubkeys);
 #else
@@ -238,7 +241,7 @@ private:
 
             mainLayout->addWidget(buttonBox);
 
-            connect(onRB, &QRadioButton::toggled, onCW, &QWidget::setEnabled);
+            connect(onRB, &QRadioButton::toggled, onCB, &QWidget::setEnabled);
             connect(inRB, &QRadioButton::toggled, inCB, &QWidget::setEnabled);
             connect(inRB, &QRadioButton::toggled, inSB, &QWidget::setEnabled);
         }
@@ -259,18 +262,18 @@ void ExpiryDialog::Private::slotInUnitChanged()
 
 void ExpiryDialog::Private::slotInAmountChanged()
 {
-    // Only modify onCW when onCW is slave:
     if (ui.inRB->isChecked()) {
-        ui.onCW->setSelectedDate(inDate());
+        ui.onCB->setDate(inDate());
     }
+    ui.inRB->setAccessibleName(accessibleValidityDuration(ui.inSB->value(), static_cast<Period>(ui.inCB->currentIndex())));
 }
 
 void ExpiryDialog::Private::slotOnDateChanged()
 {
-    // Only modify inSB/inCB when onCW is master:
     if (ui.onRB->isChecked()) {
-        ui.inSB->setValue(inAmountByDate(ui.onCW->selectedDate()));
+        ui.inSB->setValue(inAmountByDate(ui.onCB->date()));
     }
+    ui.onRB->setAccessibleName(i18nc("Valid until DATE", "Valid until %1", Formatting::accessibleDate(ui.onCB->date())));
 }
 
 QDate ExpiryDialog::Private::inDate() const
@@ -292,11 +295,21 @@ int ExpiryDialog::Private::inAmountByDate(const QDate &selected) const
     return -1;
 }
 
+void ExpiryDialog::Private::setInitialFocus()
+{
+    if (initialFocusWasSet) {
+        return;
+    }
+    // give focus to the checked radio button
+    (void) focusFirstCheckedButton({ui.neverRB, ui.inRB, ui.onRB});
+    initialFocusWasSet = true;
+}
+
 ExpiryDialog::ExpiryDialog(Mode mode, QWidget *p)
     : QDialog{p}
     , d{new Private{mode, this}}
 {
-    setWindowTitle(i18nc("@title:window", "Change Expiry"));
+    setWindowTitle(i18nc("@title:window", "Change Validity Period"));
 }
 
 ExpiryDialog::~ExpiryDialog() = default;
@@ -306,10 +319,10 @@ void ExpiryDialog::setDateOfExpiry(const QDate &date)
     const QDate current = QDate::currentDate();
     if (date.isValid()) {
         d->ui.onRB->setChecked(true);
-        d->ui.onCW->setSelectedDate(qMax(date, current));
+        d->ui.onCB->setDate(qMax(date, current));
     } else {
         d->ui.neverRB->setChecked(true);
-        d->ui.onCW->setSelectedDate(current);
+        d->ui.onCB->setDate(current);
         d->ui.inSB->setValue(0);
     }
 }
@@ -318,7 +331,7 @@ QDate ExpiryDialog::dateOfExpiry() const
 {
     return
         d->ui.inRB->isChecked() ? d->inDate() :
-        d->ui.onRB->isChecked() ? d->ui.onCW->selectedDate() :
+        d->ui.onRB->isChecked() ? d->ui.onCB->date() :
         QDate{};
 }
 
@@ -332,4 +345,10 @@ bool ExpiryDialog::updateExpirationOfAllSubkeys() const
     return d->ui.updateSubkeysCheckBox->isChecked();
 }
 
+void ExpiryDialog::showEvent(QShowEvent *event)
+{
+    d->setInitialFocus();
+    QDialog::showEvent(event);
+}
+
 #include "moc_expirydialog.cpp"
